#!/usr/bin/perl
# Generated automatically from decode.pl.in by configure.

use lib '/usr/share/news-peruser';
use Peruser;

# Uses uudeview to decode MIME, uuencoded, xxencoded, and binhex encoded
# data in the currently-selected article or batch decode all the encoded
# data in the currently-selected newsgroup's articles. 
#
# When decoding the currently-selected article, the script will search the
# spool file for the other parts of a multiple-part posting, and if all
# parts are not found, it will offer to search the appropriate news server
# for those missing using the xpat extension to NNTP.

# Used by decode_current.
#
# Arguments: news server hostname, group name, regular expression.
# Returns: 1 on success (matches found), 0 on failure.

sub search_spool_for_missing
{
   my ( $server, $group, $name ) = @_;

   feedback( "Searching spoolfile for missing data...\n" );

   my @results = search_groups( $server, [ $group ], $name );

   return 0 if ( not $#results );

   if ( @results )
   {
      my @matches = @{ $results[ 0 ] };
      shift @matches; 
      shift @matches;

      my $idx = 0;
      my $old_ordinal = -1;

      while( @matches )
      {
         my @list = @{ shift @matches };
         my $ordinal = $list[ 0 ];

         next if ( $ordinal == $old_ordinal );
         $old_ordinal = $ordinal;

         return 0 if (( not select_article( $server, $group, $ordinal,
                                            undef, undef )) ||
                      ( not export_current_article( $idx++ )));
      }
   }

   return scalar @results;
}

# Used by decode_current.
#
# Arguments: news server hostname, group name, WILDMAT pattern.
# Returns: number of matches on server.

sub search_server_for_missing
{
   my ( $server, $group, $name ) = @_;

   my $query = "There are parts missing from $name.\n" .
               "Shall I search for them in $group on $server?\n";
   my $answer = `npyesno '$query'`;
   return 0 if ( not ( answer eq 'yes' ));

   return 0 unless ( open_server $server );

   print "\nSearching $server with xpat command for missing data...\n";
   my %results = xpat( $group, 'Subject', "*$name*" );

   if ( scalar keys %results )
   {
      if ( safe_open( \*PID, "$home/.npsummary:pid" ))
      {
         my $pid = <PID>;
         safe_close( \*PID );

         unless ( kill( 'USR2', $pid ))
         {
            feedback( "$pid could not be signalled.\nRemoving" .
               " stale $home/.npsummary:pid" );
            unlink "$home/.npsummary:pid";
            return 0;
         }
      }

      while( -e "$spooldir/.current_group:lock" ){}

      unless ( safe_open( \*SPOOL, ">>$spooldir/$server-$group" ))
      {
         disconnect();
         return 0;
      }

      unless ( safe_open( \*READ, ">>$spooldir/$server-$group:read" ))
      {
         disconnect();
         safe_close( \*SPOOL );
         return 0;
      }

      print "\nDownloading matches...\n";

      foreach ( sort keys %results )
      {
         print "\tarticle number $_: $results{ $_ }...\n";

         print Peruser::SOCKET "article $_\r\n";
         my $response = <Peruser::SOCKET>; 
         my $code = substr( $response, 0, 3 );

         if ( not( $code eq '220' ))
         {
            # No such message or message not found
            next if ( $code eq '423' || $code eq '430' );

            disconnect();
            safe_close( \*SPOOL );
            safe_close( \*READ );
            feedback( "$server responded to \"article " .
               "$_\\r\\n\" with $response" );
            return 0;
         }

         while( <Peruser::SOCKET> )
         {
            print SPOOL $_;
            last if ( $_ =~ /^.\r\n$/ );
         }

         print READ "u\n";
      }

      safe_close( \*SPOOL );
      safe_close( \*READ );
   }
   else
   {
      print "There were no matches on the server.\n";
   }

   disconnect;

   scalar keys %results;
}

# Decodes encoded data in the currently-selected article.
#
# Arguments: none.
# Returns: 1 on successful decoding of embedded binary, 0 otherwise.

sub decode_current
{
   my $path = "$home/.nptext:pid";

   $wait = 0;
   while( -e "$path:lock" )
   {
      if ( ++$wait > 10000 )
      {
         feedback( "$path is locked" );
         return 0;
      }
   }

   unless ( safe_open( \*PID, "$path" ))
   {
      feedback( "The text viewer must be running and be loaded with " .
         "an article.\n" );
      return 0;
   }

   my $pid = <PID>;
   chop $pid;
   safe_close( \*PID );

   if ( not kill( 'USR2', $pid ))
   {
      feedback( "$pid could not be signalled.\nRemoving " .
         "stale $path\n" );
      unlink "$path";
      return 0;
   }

   $path = "$spooldir/.current_article_origin";
   $wait = 0;
   while ( not -e $path )
   {
      if ( ++$wait > 10000 )
      {
         feedback( "$path doesn't exist." );
         return 0;
      }
   }

   $wait = 0;
   while( -e "$path:lock" )
   {
      if ( ++$wait > 10000 )
      {
         feedback( "$path is locked" );
         return 0;
      }
   }

   unless ( safe_open( \*FILE, $path ))
   {
      return 0;
   }

   my $success = 0;

   my $server = <FILE>;
   my $group = <FILE>;
   my $offset = <FILE>;
   my $size = <FILE>;

   safe_close( \*FILE );

   chop $server;
   chop $group;
   chop $offset;
   chop $size;

   return 0 if not select_article( $server, $group, undef, $offset, $size );
   return 0 if not export_current_article( 0 );

   my $missing = 0;
   my $name, $code, $output;
   my $dest = $home;

   if ( $viewing )
   {
      $dest .= "/.peruser_spool/tmp";
      if ( not -e $dest )
      {
         mkdir $dest, 0700;
      }
   }

   for( ; ; )
   {
      unless ( open CHILD, "uudeview -i -t -p " . 
                           "$dest $spooldir/.current_article 2>&1 |" )
      {
         feedback( "Cannot fork uudeview: $!\n" );
         return 0;
      }

      while( <CHILD> )
      {
         if ( /File successfully written to (.*)/ )
         {
            push @file, $1 if ( $viewing );
            $output .= $_;
            $success = 1;
            $missing = 0;
         }
         elsif ( /No encoded data found/ )
         {
            $missing = 0;
            $output .= $_;
            last;
         }
         elsif ( /^Found \'(.+)\' State (\d+)/ )
         {
            $output .= $_;
            $name = $1;
            $code = $2;
            ++$missing if ( $code & 1 || $code & 2 || $code & 4 );
         }
      }

      close CHILD;

      if ( $missing == 1 )
      {
         feedback( $output );
         undef $output;
         next if search_spool_for_missing( $server, $group, $name );
         ++$missing;
      }

      if ( $missing == 2 )
      {
         print "Encoded data is incomplete.\n";
         next if search_server_for_missing( $server, $group, $name );
      }

      last;
   }

   if ( length( $output ) && not $viewing )
   {
      feedback( $output );
   }

   if ( $viewing )
   {
      opendir DIR, $dest;
      @entries = readdir DIR;
      closedir DIR;

      foreach $entry ( @entries )
      {
         next if ( $entry eq '..' || $entry eq '.' );
         unlink $entry;
      }
   }
   
   $success;
}

# Batch decodes all encoded data found in the currently-selected newsgroup.
#
# Arguments: none
# Returns: number of successfully decoded binaries.

sub decode_all
{
   my $pid;

   if ( safe_open( \*PID, "$home/.npsummary:pid" ))
   {
      $pid = <PID>;
      safe_close( \*PID );

      unless ( kill( 'USR1', $pid ))
      {
         feedback( "$pid could not be signalled.\n Removing stale " .
            "$home/.npsummary:pid\n" );
         unlink "$home/.npsummary:pid";
         return 0;
      }
   }
   else
   {
      feedback( "The newsgroup summary window must be loaded " .
         "with a newsgroup.\n" );
      return 0;
   }

   if ( not -e "$spooldir/.current_group" )
   {
      sleep 1;
   }

   while( -e "$spooldir/.current_group:lock" ){}

   unless ( safe_open( \*GROUP, "$spooldir/.current_group" ))
   {
      return 0;
   }

   my $server = <GROUP>;
   my $group = <GROUP>;
   safe_close( \*GROUP );
   chop $server;
   chop $group;

   unless ( open CHILD, "uudeview -i -p $home " . 
                        "$spooldir/$server-$group 2>&1 |" )
   {
      feedback( "Cannot fork uudeview: $!\n" );
      return 0;
   }

   my $count = 0;
   my $output;

   while( <CHILD> )
   {
      if ( /No encoded data found/ )
      {
         $output .= $_;
         last;
      }

      $output .= $_ if ( /^Found.*/ );
      
      if ( /File successfully written to (.*)/ )
      {
         push @file, $1 if ( $viewing );
         ++$count;
         $output .= $_;
      }
   }

   safe_close( \*CHILD );

   if ( length( $output ))
   {
      feedback( $output );
   }

   $count;
}

# Called by view.pl script. Calls decode_current with global $viewing set,
# which causes decode_current to add filenames found to global @file array,
# which this functions writes to disk.
#
# Arguments: none
# Returns: 1 on success, 0 on failure.

sub decode_view
{
   $viewing = 1;

   if ( decode_current )
   {
      unless( safe_open( \*FILE, ">$home/.peruser4-temp" ))
      {
         return 0;
      }

      foreach ( @file )
      {
         print FILE "$_\n";
      }

      safe_close( \*FILE );
      return 1;
   }

   0;
}

if ( $ARGV[ 0 ] eq 'current' )
{
   $result = decode_current;
}
elsif ( $ARGV[ 0 ] eq 'all' )
{
   $result = decode_all;
}
elsif ( $ARGV[ 0 ] eq 'view' )
{
   $result = decode_view;
}
else
{
   feedback( "Usage: decode [current|all|view]" );
}

unlink "$spooldir/.current_article";
unlink "$spooldir/.current_article_origin";

exit( not $result );
